=== modified file 'NEWS'
--- NEWS	2009-01-31 04:17:43 +0000
+++ NEWS	2009-02-04 04:19:15 +0000
@@ -62,6 +62,11 @@
     * Rule-based preferences can now accept multiple patterns for a set of
       rules.  (Marius Kruger)
 
+    * The debugger started as a result of setting ``$BZR_PDB`` works
+      around a bug in ``pdb``, http://bugs.python.org/issue4150.  The bug
+      can cause truncated tracebacks in Python versions before 2.6.
+      (Andrew Bennetts)
+
     * VirtualVersionedFiles now implements
       ``iter_lines_added_or_present_in_keys``. This allows the creation of 
       new branches based on stacked bzr-svn branches. (#311997)

=== modified file 'bzrlib/commands.py'
--- bzrlib/commands.py	2008-12-16 07:56:29 +0000
+++ bzrlib/commands.py	2009-02-04 04:10:14 +0000
@@ -894,11 +894,32 @@
     except (KeyboardInterrupt, Exception), e:
         # used to handle AssertionError and KeyboardInterrupt
         # specially here, but hopefully they're handled ok by the logger now
-        exitcode = trace.report_exception(sys.exc_info(), sys.stderr)
+        exc_info = sys.exc_info()
+        exitcode = trace.report_exception(exc_info, sys.stderr)
         if os.environ.get('BZR_PDB'):
             print '**** entering debugger'
+            tb = exc_info[2]
             import pdb
-            pdb.post_mortem(sys.exc_traceback)
+            if sys.version_info[:2] < (2, 6):
+                # XXX: we want to do
+                #    pdb.post_mortem(tb)
+                # but because pdb.post_mortem gives bad results for tracebacks
+                # from inside generators, we do it manually.
+                # (http://bugs.python.org/issue4150, fixed in Python 2.6)
+                
+                # Setup pdb on the traceback
+                p = pdb.Pdb()
+                p.reset()
+                p.setup(tb.tb_frame, tb)
+                # Point the debugger at the deepest frame of the stack
+                p.curindex = len(p.stack) - 1
+                p.curframe = p.stack[p.curindex]
+                # Start the pdb prompt.
+                p.print_stack_entry(p.stack[p.curindex])
+                p.execRcLines()
+                p.cmdloop()
+            else:
+                pdb.post_mortem(tb)
         return exitcode
 
 

